/* filename:    lbdmh_p.g
** description: The program generates draws from the posterior
**              distribution via the random walk Metropolis Hastings Algorithm
**              
** created:     05/30/01
** modified:   
*/

new;
closeall;
cls;
outwidth 128;
library user, pgraph, lbdlib;
__output = 0;

/********************************************************
**   Configuration of the Metropolis Algorithm
**
**   TOTAL NUMBER OF DRAWS = Nblocks*Nsim
**
**   mhrun: serves as an index for the output files
**          of the Metropolis Algorithm. To create parallel
**          runs of the Markov chain, run program with different
**          values for mhrun
**
**   cc0:   The first parameter vector is drawn from a multivariate
**          normal distribution, centered at the posterior mode
**          and covariance given by the inverse Hessian. cc0 scales
**          the covariance matrix. To generate multiple runs with
**          overdispersed starting points, choose a large value.
**
**   cc:    Skales the covariance matrix of the proposal density
**          for subsequent draws. It affects the rejection rate
**          of the algorithm.
**          
*/
mhrun   = "1";
cc0     = 2;
cc      = 0.3;
nblocks = 10;
nsim    = 10000;
_bounds =
    (0.1  ~ 0.99999 )|
    (1E-5 ~ 10      )|
    (1E-5 ~ 0.99999 )|
    (1E-7 ~ 10      );


/********************************************************
**      List of Global Variables
**
**      ww, xx, xw, wiiw, xiix, xiiw, iw, ix, w_names, x_names, ti
**      nx, nw, nchar, nydummies
**      gamss_m0, gamss_v0, beta_m0, beta_v0
**      age_base, educ_base, gender_base, hours_base, xbaseline
*/
#include c:\projects\active\persist\gauss\prog_p03\confspec.g;

/********************************************************
**      Load "sufficient statistics" for panel data set
**
*/
lpath = "c:\\projects\\active\\persist\\data_p\\spec" $+ ftos(mspec,"%*.*lf",1,0);
loadm path = ^lpath wiiw, xiiw, xiix, iw, ix,
                    ww, xx, xw, w_names, x_names, ti,
                    nchar, nydummies;
nx = cols(xx);
nw = cols(ww);
"Data Matrices successfully loaded";
"";

/********************************************************
**      Configure the prior for the nuisance parameters
**
**      specify baseline ind: age_base, educ_base, gender_base, hours_base     
**                            xbaseline
**      individual effect:    beta_m0, beta_v0
**      covariates       :    gamss_m0, gamss_v0
*/
#include c:\projects\active\persist\gauss\prog_p03\confprio.g;
gamss_v0 = gamss_v0*scale_g^2;
beta_v0  = beta_v0*scale_b^2;
"Prior successfully configured";
"";

/********************************************************
**   Load Posterior Mode Parameters and Hessian
*/
lpath = "c:\\projects\\active\\persist\\gauss\\prog_p03\\para";
loadm path = ^lpath lbdmod_p, para_names, sigmult;

npara = rows(lbdmod_p);
para  = lbdmod_p;

SIGSCALE     = sigmult;
SIGPROP      = SIGSCALE*SIGSCALE';
SIGPROPINV   = inv(SIGPROP);
SIGPROPlndet = ln(det(SIGPROP));
SIGPROPdim   = npara;

/********************************************************
** Load Prior for Structural Parameters
*/
priorfile = "panelpri.out";
load path=^lpath prior[npara,3] = ^priorfile;

pmean  = prior[.,1];
pstdd  = prior[.,2];
pshape = prior[.,3];


/********************************************************
**  Print Posterior Mode Information
*/

"Parameter   | Estimate ";
outmat = para_names~lbdmod_p;
let mask[1,2] = 0 1;
let fmt[2,3] =
   "-*.*s " 10 4
   "*.*lf " 10 4;
d = printfm(outmat,(0 ~ 1),fmt);

{lnpost, lnpy} = fcn(para);

"Likelihood at Mode:        " lnpy;
"Prior*Likelihood at Mode:  " lnpost;
"";
"Press Key to Continue";
k = keyw;
cls;

/* Initialize Output files
*/
opath = "c:\\projects\\active\\persist\\gauss\\prog_p03\\results\\";
ostat = opath  $+ "mp_sta" $+ mhrun;
opara = opath  $+ "mp_par" $+ mhrun;
statname = { POST LIKE PROP PROPPOST REJECT };

create fhstat=^ostat with ^statname, 5, 8;
create fhpara=^opara with ^para_names, npara, 8;

/******************************************************************
**              Metropolis Hastings Algorithm
*/

/* Initialize PARANEW
*/
valid = 0;
do until valid == 1;
   paranew   = para + cc0*(sigscale*rndn(npara,1));
   {postnew, likenew} = fcn(paranew);
   propdens  = -0.5*SIGPROPDIM*ln(2*pi) - 0.5*SIGPROPlndet - 0.5*SIGPROPDIM*ln(cc0^2)
               -0.5*(paranew - para)'*SIGPROPINV*(paranew - para)/cc0^2;
   if postnew > -1000;
      valid = 1;
   endif;
endo;   


iblock = 1;
do until iblock > nblocks;

   parasim = zeros(nsim,npara);
   likesim = zeros(nsim,1);
   postsim = zeros(nsim,1);
   rej     = zeros(nsim,1);
   propsim = zeros(nsim,1);
   proppostsim = zeros(nsim,1);

   if iblock == 1;
      j = 2;
      likesim[1,1] = likenew;
      postsim[1,1] = postnew;
      propsim[1,1] = propdens;
      proppostsim[1,1] = postnew;
      parasim[1,.] = paranew';
      postold      = postnew;
      likeold      = likenew;
      paraold      = paranew;     
   else;
      j=1;
   endif;      

   do until j > nsim;

     paranew          = paraold + cc*(sigscale*rndn(npara,1));
     {postnew, likenew} = fcn(paranew);
     propdens         = -0.5*SIGPROPDIM*ln(2*pi) - 0.5*SIGPROPlndet - 0.5*SIGPROPDIM*ln(cc^2)
                        -0.5*(paranew - paraold)'*SIGPROPINV*(paranew - paraold)/cc^2;
     propsim[j,1]     = propdens;
     proppostsim[j,1] = postnew;

     r = minc(1 | exp( postnew - postold));
     if rndu(1,1) < r;
       /* Accept proposed jump
       */
       postsim[j,1] = postnew;
       likesim[j,1] = likenew;
       parasim[j,.] = paranew';
       paraold = paranew;
       postold = postnew;
       likeold = likenew;
     else;
       /* Reject proposed jump
       */
       likesim[j,1] = likeold;
       postsim[j,1] = postold;
       parasim[j,.] = paraold';
       rej[j] = 1;
     endif;

     locate 1,1;
     "Block          " iblock "of" nblocks;
     "Simulation step" j "of" nsim;
     "r / rej / perct" r rej[j] sumc(rej)/j;
     "Likelihood     " likesim[j,1];
     "Posterior      " postsim[j,1];
     "Proposal Dens  " propsim[j,1];
     "Estimated Score" lnpy+ln(1/(sumc(1/exp(likesim[1:j,1]-lnpy))/j));
     "Parameters     " parasim[j,.]; 

     j = j+1;
    endo;

    wr = writer(fhstat,postsim~likesim~propsim~proppostsim~rej);
    wr = writer(fhpara,parasim);

    iblock = iblock+1;
endo;  

closeall fhstat, fhpara;
end;

/****************************************************/
/*                 PROCEDURES                       */
/****************************************************/

proc (2) = fcn(para);
local lnpY, lnprio, lnpost, gam1_m, gam11_v, gam0_m, gam00_v, gam01_v, gam1_mT, gam11_vT;

    /* Fix parameter values here
    */
 
    if (para < _bounds[.,2]) and (para > _bounds[.,1]);

       {gam1_m, gam11_v, gam0_m, gam00_v, gam01_v} = qdiff_gam(para, gamss_m0, gamss_v0, mspec); 
       {lnpy, gam1_mT, gam11_vT} = loglhpanel( para, beta_m0, beta_v0, gam1_m, gam11_v);
       lnprio = priodens( para, pmean, pstdd, pshape);
       lnpost = lnpy + lnprio;

    else;
      
       lnpost = -1E6;
       lnpy   = -1E20;

    endif;
    
retp(lnpost,lnpy);  
endp;

