/* filename:    lbdmh.g
** description: The program generates draws from the posterior
**              distribution via the random walk Metropolis Hastings Algorithm
**              
** created:     06/01/01
** modified:   
*/

new;
closeall;
library user, pgraph, lbdlib;
cls;

/******************************************************** 
**      Specify the DSGE Model
**      Models: 1 = RBC
**              2 = LBD
**              3 = LBD + Effort
*/ 
mspec  = 1;
mprior = 1;
npara  = 12;


/******************************************************** 
**  Configure the Metropolis Algorithm
*/
mhrun   = "1";
cc0     = 3;
cc      = 0.2;
nblocks = 10;
nsim    = 10000;
opath = "c:\\projects\\active\\persist\\gauss\\prog_t03\\results";

_bounds =
    (0.1  ~ 0.99999 )|   
    (0.5  ~ 0.99999 )|
    (-0.1 ~ 0.1     )|
    (1E-5 ~ 0.99999 )|
    (1E-5 ~ 100     )|
    (1E-5 ~ 0.99999 )|
    (-1E-5 ~ 1       )|
    (-1E-5 ~ 0.99999 )|
    (-1E-5 ~ 10      )|
    (1E-5 ~ 100     )|
    (1E-7 ~ 100     )|
    (1E-7 ~ 100     );

/******************************************************** 
** Import data on output growth and inflation: series (nobs,2)
** observations from 1954:III to 1997:IV: 
**
** YY is composed of gdpq_cld and blsh_cl
*/
#include c:\projects\active\persist\Gauss\prog_t03\loaddata.g
loadm path = ^lpath para_names;


/******************************************************** 
** Load Posterior Mode Estimates
*/
lpara = lpath $+ "\\" $+ lmodel $+ lprior $+ "mode";
open fhpara = ^lpara for read;
para = readr(fhpara,npara);
closeall fhpara;

"Parameter   | Estimate ";
outmat = para_names'~para;
let mask[1,2] = 0 1;
let fmt[2,3] =
   "-*.*s " 10 4
   "*.*lf " 10 4;
d = printfm(outmat,(0 ~ 1),fmt);
{lnpost0, lnpy0} = fcn(para);

"";
"Prior*Likelihood at Mode:" lnpost0;
"Likelihood at Mode      :" lnpy0;
"";
"Press Key to Continue";
k = keyw;
cls;

/******************************************************** 
** Load multiplication matrix for parameters
*/
lmult = lpath $+ "\\" $+ lmodel $+ lprior $+ "mult";
open fhmult = ^lmult for read;
SIGSCALE   = readr( fhmult,npara );
closeall fhmult;

lhess = lpath $+ "\\" $+ lmodel $+ lprior $+ "hess";
open fhhess = ^lhess for read;
SIGPROPINV  = readr( fhhess,npara );
closeall fhhess;

/* Hessian is of reduced rank. Keep zero rows and columns and do SVD
*/
if mspec == 1;
   SIGPROPdim = 9;
   SIGSCALE[7:9,.] = zeros(3,npara);
elseif mspec == 2;
   SIGPROPdim = 11;
   SIGSCALE[9,.] = zeros(1,npara);
else;
   SIGPROPdim = 12;
endif;

{u , s, v} = svd1(SIGPROPINV);
SIGPROPlndet = 0;

i = 1;
do until i > npara;
   if i <= SIGPROPdim;
      s[i,i]    = 1/s[i,i];
      SIGPROPlndet = SIGPROPlndet + ln(s[i,i]);
   endif;
   i = i+1;
endo;

/******************************************************** 
** Initialize Output files
*/

ostat = opath $+ "\\" $+ lmodel $+ lprior $+ "sta" $+ mhrun;
opara = opath $+ "\\" $+ lmodel $+ lprior $+ "par" $+ mhrun;
statname = { POST LIKE PROP PROPPOST REJECT };

create fhstat=^ostat with ^statname, 5, 8;
create fhpara=^opara with ^para_names, npara, 8;

/******************************************************** 
**              Metropolis Hastings Algorithm
*/

/* Initialize PARANEW
*/
valid = 0;
do until valid == 1;
   paranew   = para + cc0*(sigscale*rndn(npara,1));
   {postnew, likenew} = fcn(paranew);
   propdens  = -0.5*SIGPROPDIM*ln(2*pi) - 0.5*SIGPROPlndet - 0.5*SIGPROPDIM*ln(cc0^2)
               -0.5*(paranew - para)'*SIGPROPINV*(paranew - para)/cc0^2;
   if postnew > -1000;
      valid = 1;
   endif;
endo;   


iblock = 1;
do until iblock > nblocks;

   parasim = zeros(nsim,npara);
   likesim = zeros(nsim,1);
   postsim = zeros(nsim,1);
   rej     = zeros(nsim,1);
   propsim = zeros(nsim,1);
   proppostsim = zeros(nsim,1);

   if iblock == 1;
      j = 2;
      likesim[1,1] = likenew;
      postsim[1,1] = postnew;
      propsim[1,1] = propdens;
      proppostsim[1,1] = postnew;
      parasim[1,.] = paranew';
      postold      = postnew;
      likeold      = likenew;
      paraold      = paranew;     
   else;
      j=1;
   endif;      

   do until j > nsim;

     paranew          = paraold + cc*(sigscale*rndn(npara,1));
     {postnew, likenew} = fcn(paranew);
     propdens         = -0.5*SIGPROPDIM*ln(2*pi) - 0.5*SIGPROPlndet - 0.5*SIGPROPDIM*ln(cc^2)
                        -0.5*(paranew - paraold)'*SIGPROPINV*(paranew - paraold)/cc^2;
     propsim[j,1]     = propdens;
     proppostsim[j,1] = postnew;

     r = minc(1 | exp( postnew - postold));
     if rndu(1,1) < r;
       /* Accept proposed jump
       */
       postsim[j,1] = postnew;
       likesim[j,1] = likenew;
       parasim[j,.] = paranew';
       paraold = paranew;
       postold = postnew;
       likeold = likenew;
     else;
       /* Reject proposed jump
       */
       likesim[j,1] = likeold;
       postsim[j,1] = postold;
       parasim[j,.] = paraold';
       rej[j] = 1;
     endif;

     locate 1,1;
     "Block" iblock "of" nblocks;
     "Simulation step" j "of" nsim;
     "Rejection: r / rej / perct " r rej[j] sumc(rej)/j;
     "Likelihood     " likesim[j,1];
     "Posterior      " postsim[j,1];
     "Proposal Dens  " propsim[j,1];
     "Estimated Score" lnpy0+ln(1/(sumc(1/exp(likesim[1:j,1]-lnpy0))/j));
     "Parameters     "; parasim[j,.]; 

     j = j+1;
    endo;

    wr = writer(fhstat,postsim~likesim~propsim~proppostsim~rej);
    wr = writer(fhpara,parasim);

    iblock = iblock+1;
endo;  

closeall fhstat, fhpara;
end;

/****************************************************/
/*                 PROCEDURES                       */
/****************************************************/

proc (2) = fcn(para);
local lnpY, lnprio1, lnprio2, obsmean, obsvar, lnpost;

    /* Fix parameter values here
    */
 
    if (para < _bounds[.,2]) and (para > _bounds[.,1]);

       {lnpy, obsmean, obsvar } = evallbd( para, mspec, T0 ,YY);
       lnpy    = real(lnpy);
       lnprio1 = priodens( para, pmean, pstdd, pshape);
       lnprio2 = priomuphi( para );
       lnpost  = lnpy + real(lnprio1) + real(lnprio2);

    else;
      
       lnpost = -1E6;
       lnpy   = -1E20;

    endif;
    
retp(lnpost,lnpy);  
endp;

/***************************************************************************
*/

proc (1) = priomuphi(para);
local muphi, lnprio;
muphi = para[7:8];
if mspec > 1;
   lnprio = -ln(2*pi) - 0.5*ln(det(muphi_v0))
            - 0.5*(muphi - muphi_m0)'*inv(muphi_v0)*(muphi - muphi_m0);
else;
   lnprio = 0;
endif;
retp(lnprio);
endp;


